@@ -11,10 +11,34 @@ from betterboto import client as betterboto_client
 logger = logging.getLogger(__file__)
 
 
+def terminate_provisioned_product(prefix, service_catalog, provisioned_product_id):
+    logger.info(f"{prefix} :: about to terminate {provisioned_product_id}")
+    record_detail = service_catalog.terminate_provisioned_product(
+        ProvisionedProductId=provisioned_product_id
+    ).get('RecordDetail')
+    record_id = record_detail.get('RecordId')
+
+    logger.info(f"{prefix} :: waiting for termination of {provisioned_product_id} to complete")
+    status = "IN_PROGRESS"
+    while status == "IN_PROGRESS":
+        record_detail = service_catalog.describe_record(Id=record_id).get('RecordDetail')
+        status = record_detail.get('Status')
+        logger.info(f"{prefix} :: termination of {provisioned_product_id} current status: {status}")
+        if status != "IN_PROGRESS":
+            break
+        else:
+            time.sleep(3)
+
+    if status not in ['CREATED', 'SUCCEEDED']:
+        logger.info(yaml.safe_dump(record_detail.get('RecordErrors')))
+        raise Exception(f"Failed to terminate provisioned product: Status = {status}")
+
+
 def terminate_if_status_is_not_available(
         service_catalog, provisioned_product_name, product_id, account_id, region
 ):
-    logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: checking if should be terminated")
+    prefix = f"[{provisioned_product_name}] {account_id}:{region}"
+    logger.info(f"{prefix} :: checking if should be terminated")
     # TODO - change to name query?
     response = service_catalog.search_provisioned_products(
         Filters={'SearchQuery': [
@@ -30,12 +54,12 @@ def terminate_if_status_is_not_available(
                 provisioned_product_id = r.get('Id')
                 provisioning_artifact_id = r.get('ProvisioningArtifactId')
             elif current_status in ["UNDER_CHANGE", "PLAN_IN_PROGRESS"]:
-                logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: current status is {current_status}")
+                logger.info(f"{prefix} :: current status is {current_status}")
                 while True:
                     status = service_catalog.describe_provisioned_product(
                         Id=r.get('Id')
                     ).get('ProvisionedProductDetail').get('Status')
-                    logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: waiting to complete: {status}")
+                    logger.info(f"{prefix} :: waiting to complete: {status}")
                     time.sleep(5)
                     if status not in ["UNDER_CHANGE", "PLAN_IN_PROGRESS"]:
                         return terminate_if_status_is_not_available(
@@ -43,22 +67,9 @@ def terminate_if_status_is_not_available(
                         )
 
             elif current_status == 'ERROR':
-                logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: terminating as its status is {r.get('Status')}")
-                service_catalog.terminate_provisioned_product(
-                    ProvisionedProductId=r.get('Id')
-                )
-                logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: waiting for termination")
-                while True:
-                    response = service_catalog.search_provisioned_products(
-                        Filters={
-                            'SearchQuery': [f'name:{provisioned_product_name}']
-                        }
-                    )
-                    if len(response.get('ProvisionedProducts')) > 0:
-                        time.sleep(5)
-                    else:
-                        break
-    logger.info(f"[{provisioned_product_name}] {account_id}:{region} :: Finished waiting for termination")
+                logger.info(f"{prefix} :: terminating as its status is {r.get('Status')}")
+                terminate_provisioned_product(prefix, service_catalog, r.get('Id'))
+    logger.info(f"{prefix} :: Finished waiting for termination")
     return provisioned_product_id, provisioning_artifact_id
 
 
@@ -100,10 +111,11 @@ def provision_product(
         path_id,
         params,
         version,
+        should_use_sns,
 ):
     uid = f"[{launch_name}] {account_id}:{region}]"
     stack_name = "-".join([constants.PREFIX, account_id, region, launch_name])
-    logger.info(f"[{launch_name}] {account_id}:{region} :: Checking for an existing plan")
+    logger.info(f"{uid} :: Checking for an existing plan")
     if puppet_account_id == account_id:
         provisioned_product_plans = service_catalog.list_provisioned_product_plans_single_page().get('ProvisionedProductPlans', [])
     else:
@@ -116,7 +128,7 @@ def provision_product(
             f"{uid} :: Found existing plan, going to terminate it"
             service_catalog.delete_provisioned_product_plan(PlanId=provisioned_product_plan.get('PlanId'))
 
-    logger.info(f"[{launch_name}] {account_id}:{region} :: Creating a plan")
+    logger.info(f"{uid} :: Creating a plan")
     regional_sns_topic = f"arn:aws:sns:{region}:{puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events"
     provisioning_parameters = []
     for p in params.keys():
@@ -144,9 +156,9 @@ def provision_product(
         ],
         NotificationArns=[
             regional_sns_topic,
-        ],
+        ] if should_use_sns else [],
     )
-    logger.info(f"[{launch_name}] {account_id}:{region} :: Plan created, waiting for completion")
+    logger.info(f"{uid} :: Plan created, waiting for completion")
 
     plan_id = response.get('PlanId')
     plan_status = 'CREATE_IN_PROGRESS'
@@ -156,49 +168,50 @@ def provision_product(
             PlanId=plan_id
         )
         plan_status = response.get('ProvisionedProductPlanDetails').get('Status')
-        logger.info(f"[{launch_name}] {account_id}:{region} :: Waiting for product plan: {plan_status}")
+        logger.info(f"{uid} :: Waiting for product plan: {plan_status}")
         time.sleep(5)
 
-    if plan_status == 'CREATE_SUCCESS':
+    if plan_status in ['CREATE_SUCCESS', 'EXECUTE_SUCCESS']:
         logger.info(
-            f"[{launch_name}] {account_id}:{region} :: Plan created, "
+            f"{uid} :: Plan created, "
             f"changes: {yaml.safe_dump(response.get('ResourceChanges'))}"
         )
         if len(response.get('ResourceChanges')) == 0:
-            logger.warning(f"[{launch_name}] {account_id}:{region} :: There are no resource changes in this plan, "
+            logger.warning(f"{uid} :: There are no resource changes in this plan, "
                         f"running this anyway - your product will be marked as tainted as your CloudFormation changeset"
                         f"will fail but your product will be the correct version and in tact.")
 
 
-        logger.info(f"[{launch_name}] {account_id}:{region} :: executing changes")
+        logger.info(f"{uid} :: executing changes")
         service_catalog.execute_provisioned_product_plan(PlanId=plan_id)
-        execute_status = 'EXECUTE_IN_PROGRESS'
-        while execute_status == 'EXECUTE_IN_PROGRESS':
+        plan_execute_status = 'EXECUTE_IN_PROGRESS'
+        while plan_execute_status == 'EXECUTE_IN_PROGRESS':
             response = service_catalog.describe_provisioned_product_plan(
                 PlanId=plan_id
             )
-            logger.info(f"[{launch_name}] {account_id}:{region} :: executing changes for plan: {plan_id}")
-            execute_status = response.get('ProvisionedProductPlanDetails').get('Status')
-            logger.info(f"[{launch_name}] {account_id}:{region} :: waiting for execution to complete: {execute_status}")
+            logger.info(f"{uid} :: executing changes for plan: {plan_id}")
+            plan_execute_status = response.get('ProvisionedProductPlanDetails').get('Status')
+            logger.info(f"{uid} :: waiting for execution to complete: {plan_execute_status}")
             time.sleep(5)
 
-        if execute_status == 'CREATE_SUCCESS':
+        if plan_execute_status in ['CREATE_SUCCESS', 'EXECUTE_SUCCESS']:
             provisioned_product_id = response.get('ProvisionedProductPlanDetails').get('ProvisionProductId')
 
-            logger.info(f"[{launch_name}] {account_id}:{region} :: waiting for change to complete")
+            logger.info(f"{uid} :: waiting for change to complete")
             while True:
                 response = service_catalog.describe_provisioned_product(
                     Id=provisioned_product_id
                 )
                 logger.info(
-                    f"[{launch_name}] {account_id}:{region} :: "
+                    f"{uid} :: "
                     f"waiting for change to complete: {response.get('ProvisionedProductDetail').get('Status')}"
                 )
-                execute_status = response.get('ProvisionedProductDetail').get('Status')
-                if execute_status in ['AVAILABLE', 'TAINTED']:
+                provisioned_product_detail = response.get('ProvisionedProductDetail')
+                execute_status = provisioned_product_detail.get('Status')
+                if execute_status in ['AVAILABLE', 'TAINTED', 'EXECUTE_SUCCESS']:
                     break
                 elif execute_status ==  'ERROR':
-                    raise Exception(f"[{launch_name}] {account_id}:{region} :: Execute failed: {execute_status}")
+                    raise Exception(f"{uid} :: Execute failed: {execute_status}: {provisioned_product_detail.get('StatusMessage')}")
                 else:
                     time.sleep(5)
 
@@ -206,21 +219,28 @@ def provision_product(
             return provisioned_product_id
 
         else:
-            raise Exception(f"[{launch_name}] {account_id}:{region} :: Execute failed: {execute_status}")
+            raise Exception(f"{uid} :: Plan execute failed: {plan_execute_status}")
 
     else:
-        raise Exception(f"[{launch_name}] {account_id}:{region} :: "
-                     f"Plan failed: {response.get('ProvisionedProductPlanDetails').get('StatusMessage')}")
+        raise Exception(f"{uid} :: Plan failed: {response.get('ProvisionedProductPlanDetails').get('StatusMessage')}")
 
 
-def get_path_for_product(service_catalog, product_id):
+def get_path_for_product(service_catalog, product_id, portfolio_name):
     logger.info(f'Getting path for product {product_id}')
     response = service_catalog.list_launch_paths(ProductId=product_id)
-    if len(response.get('LaunchPathSummaries')) != 1:
-        raise Exception("Found unexpected amount of LaunchPathSummaries")
-    path_id = response.get('LaunchPathSummaries')[0].get('Id')
-    logger.info(f'Got path: {path_id} for product: {product_id}')
-    return path_id
+    if len(response.get('LaunchPathSummaries')) == 1:
+        path_id = response.get('LaunchPathSummaries')[0].get('Id')
+        logger.info(f'There is only one path: {path_id} for product: {product_id}')
+        return path_id
+    else:
+        for launch_path_summary in response.get('LaunchPathSummaries', []):
+            name = launch_path_summary.get('Name')
+            if name == portfolio_name:
+                path_id = launch_path_summary.get('Id')
+                logger.info(f'Got path: {path_id} for product: {product_id}')
+                return path_id
+
+    raise Exception("Failed to find a LaunchPath")
 
 
 def ensure_is_terminated(
@@ -238,20 +258,7 @@ def ensure_is_terminated(
 
     if r.get('Status') != "TERMINATED":
         logger.info(f"Terminating {provisioned_product_name}, its status is: {r.get('Status')}")
-        service_catalog.terminate_provisioned_product(
-            ProvisionedProductId=r.get('Id')
-        )
-        logger.info(f"Waiting for termination of {provisioned_product_name}")
-        while True:
-            response = service_catalog.search_provisioned_products(
-                Filters={
-                    'SearchQuery': [f'name:{provisioned_product_name}']
-                }
-            )
-            if len(response.get('ProvisionedProducts')) > 0:
-                time.sleep(5)
-            else:
-                break
+        terminate_provisioned_product(f"{provisioned_product_name}:{product_id}", service_catalog, r.get('Id'))
     else:
         logger.info(f"Skipping terminated launch: {provisioned_product_name}")
 
@@ -282,44 +289,38 @@ def get_provisioning_artifact_id_for(portfolio_name, product_name, version_name,
         product_id = None
         version_id = None
         portfolio_id = None
-        args = {}
-        while True:
-            response = cross_account_servicecatalog.list_accepted_portfolio_shares()
-            assert response.get('NextPageToken') is None, "Pagination not supported"
-            for portfolio_detail in response.get('PortfolioDetails'):
+
+
+        response = cross_account_servicecatalog.list_accepted_portfolio_shares()
+        assert response.get('NextPageToken') is None, "Pagination not supported"
+        for portfolio_detail in response.get('PortfolioDetails'):
+            if portfolio_detail.get('DisplayName') == portfolio_name:
+                portfolio_id = portfolio_detail.get('Id')
+                break
+
+        if portfolio_id is None:
+            response = cross_account_servicecatalog.list_portfolios()
+            for portfolio_detail in response.get('PortfolioDetails', []):
                 if portfolio_detail.get('DisplayName') == portfolio_name:
                     portfolio_id = portfolio_detail.get('Id')
                     break
 
-            if portfolio_id is None:
-                response = cross_account_servicecatalog.list_portfolios()
-                for portfolio_detail in response.get('PortfolioDetails', []):
-                    if portfolio_detail.get('DisplayName') == portfolio_name:
-                        portfolio_id = portfolio_detail.get('Id')
-                        break
+        assert portfolio_id is not None, "Could not find portfolio"
+        logger.info(f"Found portfolio: {portfolio_id}, looking for product: {product_name}")
 
-            assert portfolio_id is not None, "Could not find portfolio"
-            logger.info("Found portfolio: {}".format(portfolio_id))
+        response = cross_account_servicecatalog.search_products_as_admin_single_page(PortfolioId=portfolio_id)
+        for product_view_details in response.get('ProductViewDetails'):
+            product_view = product_view_details.get('ProductViewSummary')
+            logging.info(f"looking at {product_view.get('Name')}")
+            if product_view.get('Name') == product_name:
+                logger.info('Found product: {}'.format(product_view))
+                product_id = product_view.get('ProductId')
 
-            args['PortfolioId'] = portfolio_id
-            response = cross_account_servicecatalog.search_products_as_admin(
-                **args
-            )
-            for product_view_details in response.get('ProductViewDetails'):
-                product_view = product_view_details.get('ProductViewSummary')
-                if product_view.get('Name') == product_name:
-                    logger.info('Found product: {}'.format(product_view))
-                    product_id = product_view.get('ProductId')
-            if response.get('NextPageToken', None) is not None:
-                args['PageToken'] = response.get('NextPageToken')
-            else:
-                break
         assert product_id is not None, "Did not find product looking for"
 
-        response = cross_account_servicecatalog.list_provisioning_artifacts(
+        response = cross_account_servicecatalog.list_provisioning_artifacts_single_page(
             ProductId=product_id
         )
-        assert response.get('NextPageToken') is None, "Pagination not support"
         for provisioning_artifact_detail in response.get('ProvisioningArtifactDetails'):
             if provisioning_artifact_detail.get('Name') == version_name:
                 version_id = provisioning_artifact_detail.get('Id')
